@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "Xlib.scm", Time-stamp: <2009-01-13 14:06:51 feeley>
+;;; File: "Xlib.scm"
 
-;;; Copyright (c) 2006-2009 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 2006-2012 by Marc Feeley, All Rights Reserved.
 
 ;;; A simple interface to the X Window System Xlib library.
 
@@ -58,54 +58,6 @@ end-of-c-declare
 #include <stdio.h>
 #endif
 
-___SCMOBJ XFree_GC( void* ptr )
-{ GC p = ptr;
-#ifdef debug_free
-  printf( "XFree_GC(%p)\n", p );
-  fflush( stdout );
-#endif
-#ifdef really_free
-  XFree( p );
-#endif
-  return ___FIX(___NO_ERR);
-}
-
-___SCMOBJ XFree_Visual( void* ptr )
-{ Visual* p = ptr;
-#ifdef debug_free
-  printf( "XFree_Visual(%p)\n", p );
-  fflush( stdout );
-#endif
-#ifdef really_free
-  XFree( p );
-#endif
-  return ___FIX(___NO_ERR);
-}
-
-___SCMOBJ XFree_Display( void* ptr )
-{ Display* p = ptr;
-#ifdef debug_free
-  printf( "XFree_Display(%p)\n", p );
-  fflush( stdout );
-#endif
-#ifdef really_free
-  XFree( p );
-#endif
-  return ___FIX(___NO_ERR);
-}
-
-___SCMOBJ XFree_Screen( void* ptr )
-{ Screen* p = ptr;
-#ifdef debug_free
-  printf( "XFree_Screen(%p)\n", p );
-  fflush( stdout );
-#endif
-#ifdef really_free
-  XFree( p );
-#endif
-  return ___FIX(___NO_ERR);
-}
-
 ___SCMOBJ release_rc_XGCValues( void* ptr )
 { XGCValues* p = ptr;
 #ifdef debug_free
@@ -118,14 +70,14 @@ ___SCMOBJ release_rc_XGCValues( void* ptr )
   return ___FIX(___NO_ERR);
 }
 
-___SCMOBJ XFree_XFontStruct( void* ptr )
+___SCMOBJ XFreeFontInfo_XFontStruct( void* ptr )
 { XFontStruct* p = ptr;
 #ifdef debug_free
-  printf( "XFree_XFontStruct(%p)\n", p );
+  printf( "XFreeFontInfo_XFontStruct(%p)\n", p );
   fflush( stdout );
 #endif
 #ifdef really_free
-  XFree( p );
+  XFreeFontInfo( NULL, p, 1 );
 #endif
   return ___FIX(___NO_ERR);
 }
@@ -160,22 +112,18 @@ end-of-c-declare
 (c-define-type Bool int)
 (c-define-type Status int)
 (c-define-type GC (pointer (struct "_XGC") (GC)))
-(c-define-type GC/XFree (pointer (struct "_XGC") (GC) "XFree_GC"))
 (c-define-type Visual "Visual")
 (c-define-type Visual* (pointer Visual (Visual*)))
-(c-define-type Visual*/XFree (pointer Visual (Visual*) "XFree_Visual"))
 (c-define-type Display "Display")
 (c-define-type Display* (pointer Display (Display*)))
-(c-define-type Display*/XFree (pointer Display (Display*) "XFree_Display"))
 (c-define-type Screen "Screen")
 (c-define-type Screen* (pointer Screen (Screen*)))
-(c-define-type Screen*/XFree (pointer Screen (Screen*) "XFree_Screen"))
 (c-define-type XGCValues "XGCValues")
 (c-define-type XGCValues* (pointer XGCValues (XGCValues*)))
 (c-define-type XGCValues*/release-rc (pointer XGCValues (XGCValues*) "release_rc_XGCValues"))
 (c-define-type XFontStruct "XFontStruct")
 (c-define-type XFontStruct* (pointer XFontStruct (XFontStruct*)))
-(c-define-type XFontStruct*/XFree (pointer XFontStruct (XFontStruct*) "XFree_XFontStruct"))
+(c-define-type XFontStruct*/XFreeFontInfo (pointer XFontStruct (XFontStruct*) "XFreeFontInfo_XFontStruct"))
 (c-define-type XColor "XColor")
 (c-define-type XColor* (pointer XColor (XColor*)))
 (c-define-type XColor*/release-rc (pointer XColor (XColor*) "release_rc_XColor"))
@@ -190,7 +138,7 @@ end-of-c-declare
 
 (define XOpenDisplay
   (c-lambda (char*)        ;; display_name
-            Display*/XFree
+            Display*
             "XOpenDisplay"))
 
 (define XCloseDisplay
@@ -206,7 +154,7 @@ end-of-c-declare
 (define XScreenOfDisplay
   (c-lambda (Display*      ;; display
              int)          ;; screen_number
-            Screen*/XFree
+            Screen*
             "XScreenOfDisplay"))
 
 (define XDefaultColormapOfScreen
@@ -244,23 +192,23 @@ end-of-c-declare
 (define XDefaultVisual
   (c-lambda (Display*      ;; display
              int)          ;; screen_number
-            Visual*/XFree
+            Visual*
             "XDefaultVisual"))
 
 (define XDefaultVisualOfScreen
   (c-lambda (Screen*)      ;; screen
-            Visual*/XFree
+            Visual*
             "XDefaultVisualOfScreen"))
 
 (define XDefaultGC
   (c-lambda (Display*      ;; display
              int)          ;; screen_number
-            GC/XFree
+            GC
             "XDefaultGC"))
 
 (define XDefaultGCOfScreen
   (c-lambda (Screen*)      ;; screen
-            GC/XFree
+            GC
             "XDefaultGCOfScreen"))
 
 (define XBlackPixel
@@ -294,6 +242,14 @@ end-of-c-declare
             int
             "XMapWindow"))
 
+(define XResizeWindow
+  (c-lambda (Display*       ;; display
+             Window         ;; w
+             unsigned-int   ;; width
+             unsigned-int)  ;; height
+            int
+            "XResizeWindow"))
+
 (define XFlush
   (c-lambda (Display*)      ;; display
             int
@@ -304,9 +260,15 @@ end-of-c-declare
              Drawable       ;; d
              unsigned-long  ;; valuemask
              XGCValues*)    ;; values
-            GC/XFree
+            GC
             "XCreateGC"))
 
+(define XFreeGC
+  (c-lambda (Display*       ;; display
+             GC)            ;; gc
+            int
+            "XFreeGC"))
+
 (define XFillRectangle
   (c-lambda (Display*      ;; display
              Drawable      ;; d
@@ -539,21 +501,40 @@ end-of-c-declare
 (define XQueryFont
   (c-lambda (Display*       ;; display
              Font)          ;; font_ID
-            XFontStruct*/XFree
+            XFontStruct*/XFreeFontInfo
             "XQueryFont"))
 
+(define XFreeFontInfo
+  (c-lambda (nonnull-char-string-list ;; names
+             XFontStruct*             ;; free_info
+             int)                     ;; actual_count
+            int
+            "XFreeFontInfo"))
+
 (define XLoadFont
   (c-lambda (Display*       ;; display
              char*)         ;; name
             Font
             "XLoadFont"))
 
+(define XUnloadFont
+  (c-lambda (Display*       ;; display
+             Font)          ;; font
+            int
+            "XUnloadFont"))
+
 (define XLoadQueryFont
   (c-lambda (Display*       ;; display
              char*)         ;; name
-            XFontStruct*/XFree
+            XFontStruct*/XFreeFontInfo
             "XLoadQueryFont"))
 
+(define XFreeFont
+  (c-lambda (Display*       ;; display
+             XFontStruct*)  ;; font_struct
+            int
+            "XFreeFont"))
+
 (define XFontStruct-fid
   (c-lambda (XFontStruct*)  ;; font_struct
             Font
@@ -1006,6 +987,41 @@ end-of-c-lambda
             unsigned-int
             "___result = ___arg1->xcrossing.state;"))
 
+(define XConfigureEvent-x
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xconfigure.x;"))
+
+(define XConfigureEvent-y
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xconfigure.y;"))
+
+(define XConfigureEvent-width
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xconfigure.width;"))
+
+(define XConfigureEvent-height
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xconfigure.height;"))
+
+(define XConfigureEvent-border-width
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xconfigure.border_width;"))
+
+(define XResizeRequestEvent-width
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xresizerequest.width;"))
+
+(define XResizeRequestEvent-height
+  (c-lambda (XEvent*)       ;; XEvent box
+            int
+            "___result = ___arg1->xresizerequest.height;"))
+
 (define XLookupString
   (c-lambda (XEvent*)      ;; event_struct (XKeyEvent)
             KeySym
@@ -1025,6 +1041,7 @@ end-of-c-lambda
 (define (convert-XEvent ev)
   (and ev
        (let ((type (XAnyEvent-type ev)))
+
          (cond ((or (##fixnum.= type KeyPress)
                     (##fixnum.= type KeyRelease))
                 (##list
@@ -1047,6 +1064,7 @@ end-of-c-lambda
                  (XKeyEvent-keycode ev)
                  (XKeyEvent-same-screen ev)
                  (XLookupString ev)))
+
                ((or (##fixnum.= type ButtonPress)
                     (##fixnum.= type ButtonRelease))
                 (##list
@@ -1068,13 +1086,13 @@ end-of-c-lambda
                  (XButtonEvent-state ev)
                  (XButtonEvent-button ev)
                  (XButtonEvent-same-screen ev)))
+
                ((##fixnum.= type MotionNotify)
                 (##list
                  'XPointerMovedEvent
                  type
                  (XAnyEvent-serial ev)
                  (XAnyEvent-send-event ev)
-'
                  (XAnyEvent-display ev)
                  (XAnyEvent-window ev)
                  (XMotionEvent-root ev)
@@ -1087,6 +1105,7 @@ end-of-c-lambda
                  (XMotionEvent-state ev)
                  (XMotionEvent-is-hint ev)
                  (XMotionEvent-same-screen ev)))
+
                ((or (##fixnum.= type EnterNotify)
                     (##fixnum.= type LeaveNotify))
                 (##list
@@ -1110,6 +1129,32 @@ end-of-c-lambda
                  (XCrossingEvent-same-screen ev)
                  (XCrossingEvent-focus ev)
                  (XCrossingEvent-state ev)))
+
+               ((##fixnum.= type ConfigureNotify)
+                (##list
+                 'XConfigureEvent
+                 type
+                 (XAnyEvent-serial ev)
+                 (XAnyEvent-send-event ev)
+                 (XAnyEvent-display ev)
+                 (XAnyEvent-window ev)
+                 (XConfigureEvent-x ev)
+                 (XConfigureEvent-y ev)
+                 (XConfigureEvent-width ev)
+                 (XConfigureEvent-height ev)
+                 (XConfigureEvent-border-width ev)))
+
+               ((##fixnum.= type ResizeRequest)
+                (##list
+                 'XResizeRequestEvent
+                 type
+                 (XAnyEvent-serial ev)
+                 (XAnyEvent-send-event ev)
+                 (XAnyEvent-display ev)
+                 (XAnyEvent-window ev)
+                 (XResizeRequestEvent-width ev)
+                 (XResizeRequestEvent-height ev)))
+
                (else
                 #f)))))
 
