@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "_t-c-1.scm", Time-stamp: <2008-12-09 14:31:56 feeley>
+;;; File: "_t-c-1.scm"
 
-;;; Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2013 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
@@ -196,41 +196,58 @@
 ;; (i.e. ___FRAME_RESERVE and ___FRAME_ALIGNMENT).
 
 (define targ-frame-reserve #f)
-(set! targ-frame-reserve 1) ; no slots reserved for linking
+(set! targ-frame-reserve 3) ;; when the stack frame is transformed to a
+                            ;; heap frame, 3 extra slots are needed to
+                            ;; store the subtype object header, the link
+                            ;; to the next frame and the return address.
 
 (define targ-frame-alignment #f)
-(set! targ-frame-alignment 4) ; no alignment constraint
+(set! targ-frame-alignment 4) ;; align frame to multiple of 4 slots
 
 ;;;----------------------------------------------------------------------------
 ;;
 ;; Initialization/finalization of back-end
 
-(let ((targ (make-target 6 'c)))
+(define (targ-make-target)
+  (let ((targ (make-target 9 'c 1)))
 
-  (define (begin! info-port)
+    (define (begin! info-port)
 
-    (set! targ-info-port info-port)
+      (set! targ-info-port info-port)
 
-    (target-dump-set!              targ targ-dump)
-    (target-nb-regs-set!           targ targ-nb-gvm-regs)
-    (target-prim-info-set!         targ targ-prim-info)
-    (target-label-info-set!        targ targ-label-info)
-    (target-jump-info-set!         targ targ-jump-info)
-    (target-frame-constraints-set! targ (make-frame-constraints
-                                         targ-frame-reserve
-                                         targ-frame-alignment))
-    (target-proc-result-set!       targ (make-reg 1))
-    (target-task-return-set!       targ (make-reg 0))
-    (target-switch-testable?-set!  targ targ-switch-testable?)
+      (target-dump-set!              targ targ-dump)
+      (target-nb-regs-set!           targ targ-nb-gvm-regs)
+      (target-prim-info-set!         targ targ-prim-info)
+      (target-label-info-set!        targ targ-label-info)
+      (target-jump-info-set!         targ targ-jump-info)
+      (target-frame-constraints-set! targ (make-frame-constraints
+                                           targ-frame-reserve
+                                           targ-frame-alignment))
+      (target-proc-result-set!       targ (make-reg 1))
+      (target-task-return-set!       targ (make-reg 0))
+      (target-switch-testable?-set!  targ targ-switch-testable?)
+      (target-eq-testable?-set!      targ targ-eq-testable?)
+      (target-object-type-set!       targ targ-object-type)
+      (target-file-extension-set!    targ (targ-preferred-c-file-extension))
 
-    #f)
+      #f)
 
-  (define (end!)
-    #f)
+    (define (end!)
+      #f)
 
-  (target-begin!-set! targ begin!)
-  (target-end!-set! targ end!)
-  (target-add targ))
+    (target-begin!-set! targ begin!)
+    (target-end!-set! targ end!)
+
+    (targ-prim-proc-table-set! targ (make-vector 403 '()))
+
+    targ))
+
+(define (targ-prim-proc-table x)        (vector-ref x 16))
+(define (targ-prim-proc-table-set! x y) (vector-set! x 16 y))
+
+(define targ-target (targ-make-target))
+
+(target-add targ-target)
 
 (define targ-info-port #f)
 
@@ -238,26 +255,23 @@
 ;;
 ;; Primitive procedure database
 
-(define targ-prim-proc-table
-  (make-vector 403 '()))
-
 (define (targ-prim-proc-add! x)
   (let* ((sym (string->canonical-symbol (car x)))
          (index (modulo (symbol-hash sym) 403)))
     (vector-set!
-     targ-prim-proc-table
+     (targ-prim-proc-table targ-target)
      index
      (cons
       (cons
        sym
        (apply make-proc-obj (car x) #f #t #f (cdr x)))
-      (vector-ref targ-prim-proc-table index)))))
+      (vector-ref (targ-prim-proc-table targ-target) index)))))
 
 (for-each targ-prim-proc-add! prim-procs)
 
 (define (targ-prim-info name)
   (let ((index (modulo (symbol-hash name) 403)))
-    (let ((x (assq name (vector-ref targ-prim-proc-table index))))
+    (let ((x (assq name (vector-ref (targ-prim-proc-table targ-target) index))))
       (if x (cdr x) #f))))
 
 (define (targ-get-prim-info name)
@@ -268,13 +282,27 @@
         "targ-get-prim-info, unknown primitive:" name))))
 
 (define (targ-switch-testable? obj)
-  (targ-testable-with-eq? obj))
+  (targ-eq-testable? obj))
+
+(define (targ-eq-testable? obj)
+  (or (symbol-object? obj)
+      (keyword-object? obj)
+      (memq (targ-obj-type obj)
+            '(boolean null absent unused deleted void eof optional
+              key rest
+              fixnum char))))
+
+(define (targ-object-type obj)
+  (let ((t (targ-obj-type obj)))
+    (if (eq? t 'subtyped)
+        (targ-obj-subtype obj)
+        t)))
 
 ;;;----------------------------------------------------------------------------
 ;;
 ;; Dumping of a compilation module
 
-(define (targ-dump procs output output-root c-intf script-line options)
+(define (targ-dump procs output c-intf script-line options)
   (let ((c-decls (c-intf-decls c-intf))
         (c-procs (c-intf-procs c-intf))
         (c-inits (c-intf-inits c-intf))
@@ -286,14 +314,6 @@
     (c-intf-objs-set! c-intf '())
 
     (set! targ-track-scheme-option? compiler-option-track-scheme)
-    (set! targ-debug-location-option? compiler-option-debug-location)
-    (set! targ-debug-source-option? compiler-option-debug-source)
-    (set! targ-debug-environments-option? compiler-option-debug-environments)
-    (if compiler-option-debug
-        (begin
-          (set! targ-debug-location-option? #t)
-          (set! targ-debug-source-option? #t)
-          (set! targ-debug-environments-option? #t)))
 
     (targ-heap-begin!)
 
@@ -321,8 +341,7 @@
       (newline targ-info-port))
 
     (targ-heap-dump
-      (or output
-          (string-append output-root ".c"))
+      output
       (proc-obj-name (car procs))
       c-decls
       c-inits
@@ -331,15 +350,9 @@
 
     (targ-heap-end!)
 
-    (set! targ-track-scheme-option? #f)
-    (set! targ-debug-location-option? #f)
-    (set! targ-debug-source-option? #f)
-    (set! targ-debug-environments-option? #f)))
+    (set! targ-track-scheme-option? #f)))
 
 (define targ-track-scheme-option? #f)
-(define targ-debug-location-option? #f)
-(define targ-debug-source-option? #f)
-(define targ-debug-environments-option? #f)
 
 (define targ-tree-shake? #f)
 (set! targ-tree-shake? #f) ;; no tree shaking
@@ -620,13 +633,11 @@
            ((procedure)
             (targ-use-prc obj #f))
            ((symbol)
-            (list "SYM"
-                  (targ-use-sym obj)
-                  (targ-c-id-sym (symbol->string obj))))
+            (targ-use-sym obj)
+            (targ-c-id-sym2 (symbol->string obj)))
            ((keyword)
-            (list "KEY"
-                  (targ-use-key obj)
-                  (targ-c-id-key (keyword-object->string obj))))
+            (targ-use-key obj)
+            (targ-c-id-key2 (keyword-object->string obj)))
            ((flonum bigfixnum bignum ratnum cpxnum)
             (let ((x (table-ref targ-num-objs obj #f)))
               (use-subtyped-obj
@@ -657,12 +668,12 @@
       ((optional)
        '("OPTIONAL"))
       ((key)
-       '("KEY_OBJ"))
+       '("KEYOBJ"))
       ((rest)
        '("REST"))
 ;;      ((body)
-;;       '("BODY_OBJ"))
-      ((fixnum32)
+;;       '("BODYOBJ"))
+      ((fixnum)
        (list "FIX" (targ-c-s32 obj)))
       ((char)
        (list "CHR" (targ-c-char obj)))
@@ -684,10 +695,11 @@
                     y)))
               #f)))
     (if (proc-obj-primitive? proc)
-      (let ((i (targ-use-glo (string->symbol name) supply?)))
+      (begin
+        (targ-use-glo (string->symbol name) supply?)
         (if p
-          p
-          (list 'prm i (targ-c-id-glo name))))
+            p
+            (targ-c-id-prm2 name)))
       p)))
 
 (define (targ-heap-ref-obj obj)
@@ -758,12 +770,12 @@
       ((optional)
        '("REF_OPTIONAL"))
       ((key)
-       '("REF_KEY_OBJ"))
+       '("REF_KEYOBJ"))
       ((rest)
        '("REF_REST"))
 ;;      ((body)
-;;       '("REF_BODY_OBJ"))
-      ((fixnum32)
+;;       '("REF_BODYOBJ"))
+      ((fixnum)
        (list "REF_FIX" obj))
       ((char)
        (list "REF_CHR" (targ-c-char obj)))
@@ -773,20 +785,20 @@
 
 ;;;----------------------------------------------------------------------------
 
-(define (targ-linker extension? inputs output output-root)
+(define (targ-linker extension? inputs output warnings?)
   (with-exception-handling
     (lambda ()
       (let* ((root
-               (or output-root
-                   (path-strip-extension output)))
-             (out
-               (or output
-                   (string-append root ".c")))
+               (path-strip-extension output))
              (name
                (string-append module-prefix
                               (path-strip-directory root)))
              (input-files
-               (map (lambda (x) (string-append x ".c")) inputs))
+               (map (lambda (x)
+                      (if (string=? (path-extension x) "")
+                          (string-append x (targ-preferred-c-file-extension))
+                          x))
+                    inputs))
              (input-infos
                (map targ-read-linker-info input-files))
              (input-mods
@@ -809,9 +821,10 @@
                           (or script-line
                               last-script-line)))
                   last-script-line))))
+
         (targ-link
           extension?
-          out
+          output
           name
           input-mods
           (if extension?
@@ -832,7 +845,10 @@
           sym-rsrc
           key-rsrc
           glo-rsrc
-          script-line)))))
+          script-line
+          warnings?)
+
+        output))))
 
 (define (targ-make-mod name mods sym-rsrc key-rsrc glo-rsrc script-line)
   (vector name mods sym-rsrc key-rsrc glo-rsrc script-line))
@@ -855,6 +871,25 @@
 (define (targ-mod-script-line module-info)
   (vector-ref module-info 5))
 
+(define targ-generated-c-file-first-line
+  (string-append "#ifdef " c-id-prefix "LINKER_INFO"))
+
+(define (targ-read-line in)
+  (let loop ((lst '()))
+    (let ((c (read-char in)))
+      (if (or (eof-object? c)
+              (char=? c #\return)
+              (char=? c #\newline))
+          (list->str (reverse lst))
+          (loop (cons c lst))))))
+
+(define (targ-generated-c-file? file)
+  (let ((in (open-input-file* file)))
+    (and in
+         (let ((first-line (targ-read-line in)))
+           (close-input-port in)
+           (string=? targ-generated-c-file-first-line first-line)))))
+
 (define (targ-read-linker-info file)
   (let ((in (open-input-file* file)))
 
@@ -862,18 +897,9 @@
       (if in (close-input-port in))
       (compiler-error (string-append msg " " file)))
 
-    (define (read-line)
-      (let loop ((lst '()))
-        (let ((c (read-char in)))
-          (if (or (eof-object? c)
-                  (char=? c #\return)
-                  (char=? c #\newline))
-            (list->str (reverse lst))
-            (loop (cons c lst))))))
-
     (if in
-      (let ((first-line (read-line)))
-        (if (string=? "#ifdef ___LINKER_INFO" first-line)
+      (let ((first-line (targ-read-line in)))
+        (if (string=? targ-generated-c-file-first-line first-line)
           (let ((linker-info (read in)))
             (if (and (pair? linker-info)
                      (= (length linker-info) 9)
@@ -955,14 +981,14 @@
 
     (targ-dump-module-info name #f #f script-line)
 
-    (targ-define-count "SYM_COUNT" (length sym-list))
-    (targ-define-count "KEY_COUNT" (length key-list))
-    (targ-define-count "GLO_COUNT" (length glo-list))
-    (targ-define-count "SUP_COUNT" (length (keep targ-glo-supplied? glo-list)))
-    (targ-define-count "CNS_COUNT" (length cns-list))
-    (targ-define-count "SUB_COUNT" (length sub-list))
-    (targ-define-count "LBL_COUNT" targ-lbl-alloc)
-    (targ-define-count "OFD_COUNT" ofd-count)
+    (targ-define-count "SYMCOUNT" (length sym-list))
+    (targ-define-count "KEYCOUNT" (length key-list))
+    (targ-define-count "GLOCOUNT" (length glo-list))
+    (targ-define-count "SUPCOUNT" (length (keep targ-glo-supplied? glo-list)))
+    (targ-define-count "CNSCOUNT" (length cns-list))
+    (targ-define-count "SUBCOUNT" (length sub-list))
+    (targ-define-count "LBLCOUNT" targ-lbl-alloc)
+    (targ-define-count "OFDCOUNT" ofd-count)
     (targ-display "#include \"gambit.h\"")
     (targ-line)
 
@@ -986,9 +1012,11 @@
           sym-rsrc
           key-rsrc
           glo-rsrc
-          script-line)
+          script-line
+          warnings?)
 
-  (targ-validate-rsrc glo-rsrc)
+  (if warnings?
+      (targ-validate-rsrc glo-rsrc))
 
   (let* ((old-sym-glo-rsrc
           (targ-union-rsrc old-sym-rsrc old-glo-rsrc))
@@ -1053,8 +1081,8 @@
   (set! targ-source-filename #f)
   (set! targ-in-macro-definition? #f)
 
-  (targ-display "#ifdef ")
-  (targ-code* '("LINKER_INFO"))
+  (targ-display targ-generated-c-file-first-line)
+  (targ-line)
 
   (targ-display "; File: ")
   (targ-display-c-string (path-strip-directory filename))
@@ -1117,39 +1145,31 @@
 
 (define (targ-dump-module-info name linkfile? extension? script-line)
 
-  (targ-display "#define ")
-  (targ-code '("VERSION "))
-  (targ-display (compiler-version))
-  (targ-line)
+  (targ-macro-definition '("VERSION")
+                         (compiler-version))
+
+  (targ-macro-definition (if linkfile?
+                             '("LINKFILE_NAME")
+                             '("MODULE_NAME"))
+                         (targ-c-string name))
+
+  (targ-macro-definition '("LINKER_ID")
+                         (targ-c-id-linker name))
 
-  (targ-display "#define ")
   (if linkfile?
-    (targ-code '("LINKFILE_NAME "))
-    (targ-code '("MODULE_NAME ")))
-  (targ-display-c-string name)
-  (targ-line)
 
-  (targ-display "#define ")
-  (targ-code '("LINKER_ID "))
-  (targ-code (targ-c-id-linker name))
-  (targ-line)
+      (targ-macro-definition (if extension?
+                                 '("INCREMENTAL_LINKFILE")
+                                 '("FLAT_LINKFILE"))
+                             #f)
 
-  (targ-display "#define ")
-  (if linkfile?
-    (if extension?
-      (targ-code '("INCREMENTAL_LINKFILE"))
-      (targ-code '("FLAT_LINKFILE")))
-    (begin
-      (targ-code '("MH_PROC "))
-      (targ-code (targ-c-id-host name))))
-  (targ-line)
+      (targ-macro-definition '("MH_PROC")
+                             (targ-c-id-host name)))
 
-  (targ-display "#define ")
-  (targ-code '("SCRIPT_LINE "))
-  (if script-line
-    (targ-display-c-string script-line)
-    (targ-display 0))
-  (targ-line))
+  (targ-macro-definition '("SCRIPT_LINE")
+                         (if script-line
+                             (targ-c-string script-line)
+                             0)))
 
 (define (targ-dump-sym-key-glo-comp sym-list key-list glo-list)
 
@@ -1189,6 +1209,14 @@
                           (targ-c-id-sym name)
                           (targ-c-string name))))))
 
+  (targ-dump-section #f #f #f sym-list
+    (lambda (i s)
+      (let ((name (symbol->string (car s))))
+        (targ-macro-definition (targ-c-id-sym2 name)
+                               (list "SYM"
+                                     i
+                                     (targ-c-id-sym name))))))
+
   (targ-dump-section "BEGIN_KEY1" "END_KEY1" #f key-list
     (lambda (i k)
       (targ-cell-set! (cdr k) i)
@@ -1198,15 +1226,37 @@
                           (targ-c-id-key name)
                           (targ-c-string name))))))
 
-  (targ-dump-section "BEGIN_GLO" "END_GLO" #f
-    (append (keep targ-glo-supplied? glo-list)
-            (keep targ-glo-not-supplied? glo-list))
-    (lambda (i g)
-      (targ-cell-set! (cadr g) i)
-      (let ((name (symbol->string (car g))))
-        (targ-code* (list "DEF_GLO"
-                          i
-                          (targ-c-string name)))))))
+  (targ-dump-section #f #f #f key-list
+    (lambda (i k)
+      (let ((name (keyword-object->string (car k))))
+        (targ-macro-definition (targ-c-id-key2 name)
+                               (list "KEY"
+                                     i
+                                     (targ-c-id-key name))))))
+
+  (let ((glos
+         (append (keep targ-glo-supplied? glo-list)
+                 (keep targ-glo-not-supplied? glo-list))))
+
+    (targ-dump-section "BEGIN_GLO" "END_GLO" #f glos
+      (lambda (i g)
+        (targ-cell-set! (cadr g) i)
+        (let ((name (symbol->string (car g))))
+          (targ-code* (list "DEF_GLO"
+                            i
+                            (targ-c-string name))))))
+
+    (targ-dump-section #f #f #f glos
+      (lambda (i g)
+        (let ((name (symbol->string (car g))))
+          (targ-macro-definition (targ-c-id-glo2 name)
+                                 (list "GLO"
+                                       i
+                                       (targ-c-id-glo name)))
+          (targ-macro-definition (targ-c-id-prm2 name)
+                                 (list "PRM"
+                                       i
+                                       (targ-c-id-glo name))))))))
 
 (define (targ-dump-sym-key-glo-link
           old-sym-glo-rsrc
@@ -1265,12 +1315,7 @@
 
 (define (targ-define-count name n)
   (if (not (= n 0))
-    (begin
-      (targ-display "#define ")
-      (targ-code (list name))
-      (targ-display " ")
-      (targ-display n)
-      (targ-line))))
+      (targ-macro-definition (list name) n)))
 
 (define (targ-define-prefix var)
   (targ-display "#undef ")
@@ -1283,7 +1328,7 @@
   (if (not (null? lst))
     (begin
       (targ-line)
-      (targ-code* (list begin-name))
+      (if begin-name (targ-code* (list begin-name)))
       (if comma? (targ-display " "))
       (dump 0 (car lst))
       (let loop ((i 1) (lst (cdr lst)))
@@ -1292,7 +1337,7 @@
             (if comma? (targ-display ","))
             (dump i (car lst))
             (loop (+ i 1) (cdr lst)))))
-      (targ-code* (list end-name)))))
+      (if end-name (targ-code* (list end-name))))))
 
 (define (targ-dump-cns objs)
   (targ-dump-section "BEGIN_CNS" "END_CNS" #t objs
@@ -1511,12 +1556,7 @@
                 (p (cdr obj))
                 (c-name (proc-obj-c-name proc)))
            (if c-name
-             (begin
-               (targ-display "#define ")
-               (targ-code (cons 'c-lbl- c-name))
-               (targ-display " ")
-               (targ-code (caddr p))
-               (targ-line)))))
+               (targ-macro-definition (cons 'c-lbl- c-name) (caddr p)))))
        objs)))
 
   (if (pair? c-objs)
@@ -1525,11 +1565,7 @@
       (let loop ((i 0) (lst c-objs))
         (if (pair? lst)
           (let ((x (car lst)))
-            (targ-display "#define ")
-            (targ-code (cons 'c-obj- i))
-            (targ-display " ")
-            (targ-code x)
-            (targ-line)
+            (targ-macro-definition (cons 'c-obj- i) x)
             (loop (+ i 1) (cdr lst)))))))
 
   (if (pair? c-decls)
@@ -1875,8 +1911,12 @@
 (define targ-linker-tag "")
 (define targ-host-tag   "H_")
 (define targ-glo-tag    "G_")
+(define targ-glo2-tag   "GLO_")
+(define targ-prm2-tag   "PRM_")
 (define targ-sym-tag    "S_")
+(define targ-sym2-tag   "SYM_")
 (define targ-key-tag    "K_")
+(define targ-key2-tag   "KEY_")
 
 (define targ-port #f)
 (define targ-line-size #f)
@@ -1920,8 +1960,12 @@
         ((c-id-linker) (targ-display-c-id        targ-linker-tag tail))
         ((c-id-host)   (targ-display-c-id        targ-host-tag   tail))
         ((c-id-glo)    (targ-display-c-id        targ-glo-tag    tail))
+        ((c-id-glo2)   (targ-display-c-id        targ-glo2-tag   tail))
+        ((c-id-prm2)   (targ-display-c-id        targ-prm2-tag   tail))
         ((c-id-sym)    (targ-display-c-id        targ-sym-tag    tail))
+        ((c-id-sym2)   (targ-display-c-id        targ-sym2-tag   tail))
         ((c-id-key)    (targ-display-c-id        targ-key-tag    tail))
+        ((c-id-key2)   (targ-display-c-id        targ-key2-tag   tail))
         ((sub-name)    (targ-display-prefixed "X") (targ-code tail))
         ((glbl)        (goto-lbl (car tail) (cadr tail)))
         ((d-)          (two-heads "D_"        tail '()))
@@ -1957,7 +2001,6 @@
         ((seq)         (targ-code-seq tail))
         ((append)      (targ-code-append tail))
         ((cell)        (targ-code tail))
-        ((prm)         (one-head "PRM" tail))
         ((prc)         (one-head "PRC" tail))
         (else          (one-head head tail))))
     (if x (targ-display x))))
@@ -2044,6 +2087,17 @@
         (set! targ-current-filename targ-source-filename))))
   (targ-display-no-line-info x))
 
+(define (targ-macro-definition name value)
+  (set! targ-in-macro-definition? #t)
+  (targ-display "#define ")
+  (targ-code name)
+  (if value
+      (begin
+        (targ-display " ")
+        (targ-code value)))
+  (set! targ-in-macro-definition? #f)
+  (targ-line))
+
 (define (targ-line)
   (if targ-in-macro-definition?
     (begin
@@ -2082,12 +2136,24 @@
 (define (targ-c-id-glo name)
   (cons 'c-id-glo name))
 
+(define (targ-c-id-glo2 name)
+  (cons 'c-id-glo2 name))
+
+(define (targ-c-id-prm2 name)
+  (cons 'c-id-prm2 name))
+
 (define (targ-c-id-sym name)
   (cons 'c-id-sym name))
 
+(define (targ-c-id-sym2 name)
+  (cons 'c-id-sym2 name))
+
 (define (targ-c-id-key name)
   (cons 'c-id-key name))
 
+(define (targ-c-id-key2 name)
+  (cons 'c-id-key2 name))
+
 (define (targ-sub-name n)
   (cons 'sub-name n))
 
@@ -2210,4 +2276,21 @@
 (define (targ-name->c-id s)
   (scheme-id->c-id s))
 
+(define targ-c-file-extensions #f)
+(set! targ-c-file-extensions
+  '((".c"    . C)
+    (".C"    . C++)
+    (".cc"   . C++)
+    (".cp"   . C++)
+    (".cpp"  . C++)
+    (".CPP"  . C++)
+    (".cxx"  . C++)
+    (".c++"  . C++)
+    (".m"    . Objective-C)
+    (".M"    . Objective-C++)
+    (".mm"   . Objective-C++)))
+
+(define (targ-preferred-c-file-extension)
+  (caar targ-c-file-extensions))
+
 ;;;----------------------------------------------------------------------------
