@@ -1,7 +1,7 @@
 #ifdef ___LINKER_INFO
-; File: "_gsilib.c", produced by Gambit-C v4.4.4
+; File: "_gsilib.c", produced by Gambit-C v4.6.7
 (
-404004
+406007
 " _gsilib"
 (" _gsilib")
 (
@@ -18,14 +18,14 @@
  #f
 )
 #else
-#define ___VERSION 404004
+#define ___VERSION 406007
 #define ___MODULE_NAME " _gsilib"
 #define ___LINKER_ID ____20___gsilib
 #define ___MH_PROC ___H__20___gsilib
 #define ___SCRIPT_LINE 0
-#define ___GLO_COUNT 1
-#define ___SUP_COUNT 1
-#define ___LBL_COUNT 2
+#define ___GLOCOUNT 1
+#define ___SUPCOUNT 1
+#define ___LBLCOUNT 2
 #include "gambit.h"
 
 ___NEED_GLO(___G__20___gsilib)
@@ -34,6 +34,9 @@ ___BEGIN_GLO
 ___DEF_GLO(0," _gsilib")
 ___END_GLO
 
+#define ___GLO__20___gsilib ___GLO(0,___G__20___gsilib)
+#define ___PRM__20___gsilib ___PRM(0,___G__20___gsilib)
+
 
 #undef ___MD_ALL
 #define ___MD_ALL ___D_R0 ___D_R1
