@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "gambit#.scm", Time-stamp: <2009-06-03 12:41:47 feeley>
+;;; File: "gambit#.scm"
 
-;;; Copyright (c) 2005-2009 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 2005-2012 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -41,8 +41,26 @@ default-random-source
 ;; procedures
 abandoned-mutex-exception?
 abort
+address-info-family
+address-info-protocol
+address-info-socket-info
+address-info-socket-type
+address-info?
+address-infos
 all-bits-set?
 any-bits-set?
+append-f32vectors
+append-f64vectors
+append-s16vectors
+append-s32vectors
+append-s64vectors
+append-s8vectors
+append-strings
+append-u16vectors
+append-u32vectors
+append-u64vectors
+append-u8vectors
+append-vectors
 arithmetic-shift
 bit-count
 bit-set?
@@ -54,9 +72,11 @@ bitwise-xor
 box
 box?
 break
+call-with-input-process
 call-with-input-string
 call-with-input-u8vector
 call-with-input-vector
+call-with-output-process
 call-with-output-string
 call-with-output-u8vector
 call-with-output-vector
@@ -70,7 +90,7 @@ clear-bit-field
 close-port
 command-line
 compile-file
-compile-file-to-c
+compile-file-to-target
 condition-variable-broadcast!
 condition-variable-name
 condition-variable-signal!
@@ -139,6 +159,7 @@ f32vector-fill!
 f32vector-length
 f32vector-ref
 f32vector-set!
+f32vector-shrink!
 f32vector?
 f64vector
 f64vector->list
@@ -148,6 +169,7 @@ f64vector-fill!
 f64vector-length
 f64vector-ref
 f64vector-set!
+f64vector-shrink!
 f64vector?
 file-attributes
 file-creation-time
@@ -399,9 +421,11 @@ open-directory
 open-dummy
 open-event-queue
 open-file
+open-input-process
 open-input-string
 open-input-u8vector
 open-input-vector
+open-output-process
 open-output-string
 open-output-u8vector
 open-output-vector
@@ -476,6 +500,8 @@ read-subu8vector
 read-u8
 readtable-case-conversion?
 readtable-case-conversion?-set
+readtable-comment-handler
+readtable-comment-handler-set
 readtable-eval-allowed?
 readtable-eval-allowed?-set
 readtable-keywords-allowed?
@@ -515,6 +541,7 @@ s16vector-fill!
 s16vector-length
 s16vector-ref
 s16vector-set!
+s16vector-shrink!
 s16vector?
 s32vector
 s32vector->list
@@ -524,6 +551,7 @@ s32vector-fill!
 s32vector-length
 s32vector-ref
 s32vector-set!
+s32vector-shrink!
 s32vector?
 s64vector
 s64vector->list
@@ -533,6 +561,7 @@ s64vector-fill!
 s64vector-length
 s64vector-ref
 s64vector-set!
+s64vector-shrink!
 s64vector?
 s8vector
 s8vector->list
@@ -542,6 +571,7 @@ s8vector-fill!
 s8vector-length
 s8vector-ref
 s8vector-set!
+s8vector-shrink!
 s8vector?
 scheduler-exception-reason
 scheduler-exception?
@@ -647,18 +677,43 @@ step
 step-level-set!
 string->keyword
 string-ci=?-hash
+string-shrink!
 string=?-hash
 subf32vector
+subf32vector-fill!
+subf32vector-move!
 subf64vector
+subf64vector-fill!
+subf64vector-move!
 subs16vector
+subs16vector-fill!
+subs16vector-move!
 subs32vector
+subs32vector-fill!
+subs32vector-move!
 subs64vector
+subs64vector-fill!
+subs64vector-move!
 subs8vector
+subs8vector-fill!
+subs8vector-move!
+substring-fill!
+substring-move!
 subu16vector
+subu16vector-fill!
+subu16vector-move!
 subu32vector
+subu32vector-fill!
+subu32vector-move!
 subu64vector
+subu64vector-fill!
+subu64vector-move!
 subu8vector
+subu8vector-fill!
+subu8vector-move!
 subvector
+subvector-fill!
+subvector-move!
 symbol-hash
 system-stamp
 system-type
@@ -756,6 +811,7 @@ u16vector-fill!
 u16vector-length
 u16vector-ref
 u16vector-set!
+u16vector-shrink!
 u16vector?
 u32vector
 u32vector->list
@@ -765,6 +821,7 @@ u32vector-fill!
 u32vector-length
 u32vector-ref
 u32vector-set!
+u32vector-shrink!
 u32vector?
 u64vector
 u64vector->list
@@ -774,6 +831,7 @@ u64vector-fill!
 u64vector-length
 u64vector-ref
 u64vector-set!
+u64vector-shrink!
 u64vector?
 u8vector
 u8vector->list
@@ -784,6 +842,7 @@ u8vector-fill!
 u8vector-length
 u8vector-ref
 u8vector-set!
+u8vector-shrink!
 u8vector?
 unbound-global-exception-code
 unbound-global-exception-rte
@@ -826,6 +885,7 @@ user-info?
 user-name
 vector-append
 vector-copy
+vector-shrink!
 void
 will-execute!
 will-testator
@@ -833,10 +893,12 @@ will?
 with-exception-catcher
 with-exception-handler
 with-input-from-port
+with-input-from-process
 with-input-from-string
 with-input-from-u8vector
 with-input-from-vector
 with-output-to-port
+with-output-to-process
 with-output-to-string
 with-output-to-u8vector
 with-output-to-vector
