@@ -1,6 +1,6 @@
-/* File: "os.c", Time-stamp: <2009-06-04 18:57:32 feeley> */
+/* File: "os.c" */
 
-/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2013 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -21,6 +21,8 @@
  *  - dynamic loading
  *  - dynamic C compilation
  *  - floating point environment setup
+ *  - processor count
+ *  - processor cache size
  *  - virtual memory statistics
  *  - filesystem path expansion
  *  - formatting of source code position
@@ -28,7 +30,7 @@
  */
 
 #define ___INCLUDED_FROM_OS
-#define ___VERSION 404004
+#define ___VERSION 406007
 #include "gambit.h"
 
 #include "os_base.h"
@@ -90,6 +92,267 @@ void ___enable_os_interrupts ___PVOID
 }
 
 
+/*---------------------------------------------------------------------------*/
+
+/* Processor information. */
+
+int ___processor_count ___PVOID
+{
+  int nb_processors = 0;
+
+#ifdef USE_sysconf
+
+#ifdef _SC_NPROCESSORS_ONLN
+#define OP_SC_NPROCESSORS _SC_NPROCESSORS_ONLN
+#else
+#ifdef _SC_NPROCESSORS_CONF
+#define OP_SC_NPROCESSORS _SC_NPROCESSORS_CONF
+#endif
+#endif
+
+#endif
+
+#ifdef OP_SC_NPROCESSORS
+
+  nb_processors = sysconf (OP_SC_NPROCESSORS);
+
+#else
+
+#ifdef USE_sysctl
+
+#ifdef CTL_HW
+#ifdef HW_AVAILCPU
+#define OP_NB_CPU HW_AVAILCPU
+#else
+#ifdef HW_NCPU
+#define OP_NB_CPU HW_NCPU
+#endif
+#endif
+#endif
+
+#ifdef OP_NB_CPU
+
+  size_t n = 0;
+  size_t sizeof_n = sizeof(n);
+  int mib[2];
+
+  mib[0] = CTL_HW;
+  mib[1] = OP_NB_CPU;
+
+  if (sysctl (mib, 2, &n, &sizeof_n, NULL, 0) == 0) {
+    nb_processors = n;
+  }
+
+#endif
+
+#endif
+
+#endif
+
+#ifdef USE_GetSystemInfo
+
+ SYSTEM_INFO si;
+
+ GetSystemInfo (&si);
+
+ nb_processors = si.dwNumberOfProcessors;
+
+#endif
+
+  return nb_processors;
+}
+
+
+int ___processor_cache_size
+   ___P((___BOOL instruction_cache,
+         int level),
+        (instruction_cache,
+         level)
+___BOOL instruction_cache;
+int level;)
+{
+  int cache_size = 0;
+
+#ifdef USE_sysconf
+
+  {
+
+  static struct { int name; int level; int kind; } sysconf_info[] = {
+
+#ifdef _SC_LEVEL1_DCACHE_SIZE
+    { _SC_LEVEL1_DCACHE_SIZE, 1, 1 },
+#endif
+#ifdef _SC_LEVEL1_ICACHE_SIZE
+    { _SC_LEVEL1_ICACHE_SIZE, 1, 2 },
+#endif
+#ifdef _SC_LEVEL1_CACHE_SIZE
+    { _SC_LEVEL1_CACHE_SIZE,  1, 3 },
+#endif
+
+#ifdef _SC_LEVEL2_DCACHE_SIZE
+    { _SC_LEVEL2_DCACHE_SIZE, 2, 1 },
+#endif
+#ifdef _SC_LEVEL2_ICACHE_SIZE
+    { _SC_LEVEL2_ICACHE_SIZE, 2, 2 },
+#endif
+#ifdef _SC_LEVEL2_CACHE_SIZE
+    { _SC_LEVEL2_CACHE_SIZE,  2, 3 },
+#endif
+
+#ifdef _SC_LEVEL3_DCACHE_SIZE
+    { _SC_LEVEL3_DCACHE_SIZE, 3, 1 },
+#endif
+#ifdef _SC_LEVEL3_ICACHE_SIZE
+    { _SC_LEVEL3_ICACHE_SIZE, 3, 2 },
+#endif
+#ifdef _SC_LEVEL3_CACHE_SIZE
+    { _SC_LEVEL3_CACHE_SIZE,  3, 3 },
+#endif
+
+#ifdef _SC_LEVEL4_DCACHE_SIZE
+    { _SC_LEVEL4_DCACHE_SIZE, 4, 1 },
+#endif
+#ifdef _SC_LEVEL4_ICACHE_SIZE
+    { _SC_LEVEL4_ICACHE_SIZE, 4, 2 },
+#endif
+#ifdef _SC_LEVEL4_CACHE_SIZE
+    { _SC_LEVEL4_CACHE_SIZE,  4, 3 },
+#endif
+
+      { 0, 0, 0 }
+  };
+
+  int i = 0;
+
+  while (sysconf_info[i].kind != 0) {
+
+    if ((level == 0 || level == sysconf_info[i].level) &&
+        (sysconf_info[i].kind & (1<<instruction_cache))) {
+
+      int size = sysconf (sysconf_info[i].name);
+
+      if (level != 0) {
+        cache_size = size;
+        break;
+      }
+
+      if (size > cache_size) {
+        cache_size = size;
+      }
+    }
+
+    i++;
+  }
+
+  if (cache_size != 0) {
+    return cache_size;
+  }
+
+  }
+
+#endif
+
+#ifdef USE_sysctl
+
+#ifdef CTL_HW
+
+  {
+
+  static struct { int name; int level; int kind; } sysctl_info[] = {
+
+#ifdef HW_L1DCACHESIZE
+    { HW_L1DCACHESIZE, 1, 1 },
+#endif
+#ifdef HW_L1ICACHESIZE
+    { HW_L1ICACHESIZE, 1, 2 },
+#endif
+#ifdef HW_L1CACHESIZE
+    { HW_L1CACHESIZE,  1, 3 },
+#endif
+
+#ifdef HW_L2DCACHESIZE
+    { HW_L2DCACHESIZE, 2, 1 },
+#endif
+#ifdef HW_L2ICACHESIZE
+    { HW_L2ICACHESIZE, 2, 2 },
+#endif
+#ifdef HW_L2CACHESIZE
+    { HW_L2CACHESIZE,  2, 3 },
+#endif
+
+#ifdef HW_L3DCACHESIZE
+    { HW_L3DCACHESIZE, 3, 1 },
+#endif
+#ifdef HW_L3ICACHESIZE
+    { HW_L3ICACHESIZE, 3, 2 },
+#endif
+#ifdef HW_L3CACHESIZE
+    { HW_L3CACHESIZE,  3, 3 },
+#endif
+
+#ifdef HW_L4DCACHESIZE
+    { HW_L4DCACHESIZE, 4, 1 },
+#endif
+#ifdef HW_L4ICACHESIZE
+    { HW_L4ICACHESIZE, 4, 2 },
+#endif
+#ifdef HW_L4CACHESIZE
+    { HW_L4CACHESIZE,  4, 3 },
+#endif
+
+      { 0, 0, 0 }
+  };
+
+  int i = 0;
+
+  while (sysctl_info[i].kind != 0) {
+
+    if ((level == 0 || level == sysctl_info[i].level) &&
+        (sysctl_info[i].kind & (1<<instruction_cache))) {
+
+      size_t size = 0;
+      size_t sizeof_size = sizeof(size);
+      int mib[2];
+
+      mib[0] = CTL_HW;
+      mib[1] = sysctl_info[i].name;
+
+      if (sysctl (mib, 2, &size, &sizeof_size, NULL, 0) == 0) {
+
+        if (level != 0) {
+          cache_size = size;
+          break;
+        }
+
+        if (size > cache_size) {
+          cache_size = size;
+        }
+      }
+    }
+
+    i++;
+  }
+
+  if (cache_size != 0) {
+    return cache_size;
+  }
+
+  }
+
+#endif
+
+#endif
+
+#ifdef USE_WIN32
+
+  /* TODO: use GetLogicalProcessorInformation */
+
+#endif
+
+  return cache_size;
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 /* Virtual memory statistics. */
@@ -168,6 +431,149 @@ ___BOOL pinpoint;)
 #define USE_IPV6
 #endif
 
+
+___HIDDEN int network_family_decode
+   ___P((int family),
+        (family)
+int family;)
+{
+  switch (family)
+    {
+#ifdef PF_INET
+    case -1:
+      return PF_INET;
+#endif
+
+#ifdef PF_INET6
+    case -2:
+      return PF_INET6;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_family_encode
+   ___P((int family),
+        (family)
+int family;)
+{
+  switch (family)
+    {
+#ifdef PF_INET
+    case PF_INET:
+      return ___FIX(-1);
+#endif
+
+#ifdef PF_INET6
+    case PF_INET6:
+      return ___FIX(-2);
+#endif
+    }
+
+  return ___FIX(family);
+}
+
+
+___HIDDEN int network_socktype_decode
+   ___P((int socktype),
+        (socktype)
+int socktype;)
+{
+  switch (socktype)
+    {
+#ifdef SOCK_STREAM
+    case -1:
+      return SOCK_STREAM;
+#endif
+
+#ifdef SOCK_DGRAM
+    case -2:
+      return SOCK_DGRAM;
+#endif
+
+#ifdef SOCK_RAW
+    case -3:
+      return SOCK_RAW;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_socktype_encode
+   ___P((int socktype),
+        (socktype)
+int socktype;)
+{
+  switch (socktype)
+    {
+#ifdef SOCK_STREAM
+    case SOCK_STREAM:
+      return ___FIX(-1);
+#endif
+
+#ifdef SOCK_DGRAM
+    case SOCK_DGRAM:
+      return ___FIX(-2);
+#endif
+
+#ifdef SOCK_RAW
+    case SOCK_RAW:
+      return ___FIX(-3);
+#endif
+    }
+
+  return ___FIX(socktype);
+}
+
+
+___HIDDEN int network_protocol_decode
+   ___P((int protocol),
+        (protocol)
+int protocol;)
+{
+  switch (protocol)
+    {
+#ifdef IPPROTO_UDP
+    case -1:
+      return IPPROTO_UDP;
+#endif
+
+#ifdef IPPROTO_TCP
+    case -2:
+      return IPPROTO_TCP;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_protocol_encode
+   ___P((int protocol),
+        (protocol)
+int protocol;)
+{
+  switch (protocol)
+    {
+#ifdef IPPROTO_UDP
+    case IPPROTO_UDP:
+      return ___FIX(-1);
+#endif
+
+#ifdef IPPROTO_TCP
+    case IPPROTO_TCP:
+      return ___FIX(-2);
+#endif
+    }
+
+  return ___FIX(protocol);
+}
+
+
 ___SCMOBJ ___SCMOBJ_to_in_addr
    ___P((___SCMOBJ addr,
          struct in_addr *ia,
@@ -377,7 +783,7 @@ int arg_num;)
           return addr;
         }
 
-      ___FIELD(result,1) = ___FIX(sa_in->sin_family);
+      ___FIELD(result,1) = network_family_encode (sa_in->sin_family);
       ___FIELD(result,2) = ___FIX(ntohs (sa_in->sin_port));
       ___FIELD(result,3) = addr;
       ___release_scmobj (addr);
@@ -394,7 +800,7 @@ int arg_num;)
           return addr;
         }
 
-      ___FIELD(result,1) = ___FIX(sa_in6->sin6_family);
+      ___FIELD(result,1) = network_family_encode (sa_in6->sin6_family);
       ___FIELD(result,2) = ___FIX(ntohs (sa_in6->sin6_port));
       ___FIELD(result,3) = addr;
       ___release_scmobj (addr);
@@ -472,6 +878,193 @@ int arg_num;)
 
 /* Access to host information. */
 
+#ifdef USE_getaddrinfo
+
+___HIDDEN int ai_flags_decode
+   ___P((int flags),
+        (flags)
+int flags;)
+{
+  int ai_flags = 0;
+
+#ifdef AI_PASSIVE
+  if (flags & 1)
+    ai_flags |= AI_PASSIVE;
+#endif
+
+#ifdef AI_CANONNAME
+  if (flags & 2)
+    ai_flags |= AI_CANONNAME;
+#endif
+
+#ifdef AI_NUMERICHOST
+  if (flags & 4)
+    ai_flags |= AI_NUMERICHOST;
+#endif
+
+#ifdef AI_NUMERICSERV
+  if (flags & 8)
+    ai_flags |= AI_NUMERICSERV;
+#endif
+
+#ifdef AI_ALL
+  if (flags & 16)
+    ai_flags |= AI_ALL;
+#endif
+
+#ifdef AI_ADDRCONFIG
+  if (flags & 32)
+    ai_flags |= AI_ADDRCONFIG;
+#endif
+
+#ifdef AI_V4MAPPED
+  if (flags & 64)
+    ai_flags |= AI_V4MAPPED;
+#endif
+
+  return ai_flags;
+}
+
+#endif
+
+
+___SCMOBJ ___os_address_infos
+   ___P((___SCMOBJ host,
+         ___SCMOBJ serv,
+         ___SCMOBJ flags,
+         ___SCMOBJ family,
+         ___SCMOBJ socktype,
+         ___SCMOBJ protocol),
+        (host,
+         serv,
+         flags,
+         family,
+         socktype,
+         protocol)
+___SCMOBJ host;
+___SCMOBJ serv;
+___SCMOBJ flags;
+___SCMOBJ family;
+___SCMOBJ socktype;
+___SCMOBJ protocol;)
+{
+#ifndef USE_getaddrinfo
+
+  return ___FIX(___UNIMPL_ERR);
+
+#endif
+
+#ifdef USE_getaddrinfo
+
+  ___SCMOBJ e;
+  ___SCMOBJ vect;
+  ___SCMOBJ lst;
+  ___SCMOBJ tail;
+  ___SCMOBJ x;
+  ___SCMOBJ p;
+  int i;
+  char *chost = 0;
+  char *cserv = 0;
+
+  struct addrinfo hints, *res, *res0;
+  int code;
+
+  if ((e = ___SCMOBJ_to_CHARSTRING (host, &chost, 1))
+      != ___FIX(___NO_ERR))
+    return e;
+
+  if ((e = ___SCMOBJ_to_CHARSTRING (serv, &cserv, 2))
+      != ___FIX(___NO_ERR))
+    {
+      ___release_string (chost);
+      return e;
+    }
+
+  memset (&hints, 0, sizeof (hints));
+
+  hints.ai_flags    = ai_flags_decode (___INT(flags));
+  hints.ai_family   = network_family_decode (___INT(family));
+  hints.ai_socktype = network_socktype_decode (___INT(socktype));
+  hints.ai_protocol = network_protocol_decode (___INT(protocol));
+
+  code = getaddrinfo (chost, cserv, &hints, &res0);
+
+  if (code != 0)
+    {
+      e = err_code_from_gai_code (code);
+      ___release_string (chost);
+      ___release_string (cserv);
+      return e;
+    }
+
+  lst = ___NUL;
+  tail = ___FAL;
+
+  for (res = res0; res != NULL; res = res->ai_next)
+    {
+      x = ___sockaddr_to_SCMOBJ (res->ai_addr,
+                                 res->ai_addrlen,
+                                 ___RETURN_POS);
+
+      if (___FIXNUMP(x))
+        {
+          ___release_scmobj (lst);
+          freeaddrinfo (res0);
+          return x;
+        }
+
+      if (x != ___FAL)
+        {
+          vect = ___make_vector (5, ___FAL, ___STILL);
+
+          if (___FIXNUMP(vect))
+            {
+              ___release_scmobj (x);
+              ___release_scmobj (lst);
+              freeaddrinfo (res0);
+              return ___FIX(___CTOS_HEAP_OVERFLOW_ERR+___RETURN_POS);
+            }
+
+          ___FIELD(vect,1) = network_family_encode (res->ai_family);
+          ___FIELD(vect,2) = network_socktype_encode (res->ai_socktype);
+          ___FIELD(vect,3) = network_protocol_encode (res->ai_protocol);
+          ___FIELD(vect,4) = x;
+
+          ___release_scmobj (x);
+
+          p = ___make_pair (vect, ___NUL, ___STILL);
+
+          ___release_scmobj (vect);
+
+          if (___FIXNUMP(p))
+            {
+              ___release_scmobj (lst);
+              freeaddrinfo (res0);
+              return ___FIX(___CTOS_HEAP_OVERFLOW_ERR+___RETURN_POS);
+            }
+
+          if (lst == ___NUL)
+            lst = p;
+          else
+            ___SETCDR(tail,p);
+
+          tail = p;
+        }
+    }
+
+  ___release_scmobj (lst);
+
+  freeaddrinfo (res0);
+
+  ___release_string (chost);
+  ___release_string (cserv);
+
+  return lst;
+
+#endif
+}
+
+
 ___SCMOBJ ___os_host_info
    ___P((___SCMOBJ host),
         (host)
@@ -1155,7 +1748,7 @@ ___SCMOBJ chase;)
               0))
       == ___FIX(___NO_ERR))
     {
-      struct ___stat s;
+      ___struct_stat s;
 
       if (((chase == ___FAL)
            ? ___lstat (___CAST(___STRING_TYPE(___INFO_PATH_CE_SELECT),cpath), &s)
@@ -1781,6 +2374,49 @@ char *___os_configure_command_string ___PVOID
 }
 
 
+/*---------------------------------------------------------------------------*/
+
+/* C compilation environment information. */
+
+
+#ifndef ___OBJ_EXTENSION
+#define ___OBJ_EXTENSION ".obj"
+#endif
+
+#ifndef ___EXE_EXTENSION
+#define ___EXE_EXTENSION ".exe"
+#endif
+
+#ifndef ___BAT_EXTENSION
+#define ___BAT_EXTENSION ".bat"
+#endif
+
+
+___HIDDEN char *os_obj_extension_string = ___OBJ_EXTENSION;
+
+___HIDDEN char *os_exe_extension_string = ___EXE_EXTENSION;
+
+___HIDDEN char *os_bat_extension_string = ___BAT_EXTENSION;
+
+
+char *___os_obj_extension_string ___PVOID
+{
+  return os_obj_extension_string;
+}
+
+
+char *___os_exe_extension_string ___PVOID
+{
+  return os_exe_extension_string;
+}
+
+
+char *___os_bat_extension_string ___PVOID
+{
+  return os_bat_extension_string;
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 
