@@ -1,6 +1,6 @@
-/* File: "os.h", Time-stamp: <2009-06-04 19:30:18 feeley> */
+/* File: "os.h" */
 
-/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2013 by Marc Feeley, All Rights Reserved. */
 
 #ifndef ___OS_H
 #define ___OS_H
@@ -91,11 +91,18 @@
 
 #ifdef USE_POSIX
 
-#define USE_environ
 #define USE_open
 
 /* Select features based on availability */
 
+#ifdef HAVE_ENVIRON
+#define USE_environ
+#else
+#ifdef HAVE__NSGETENVIRON
+#define USE_environ
+#endif
+#endif
+
 #ifdef HAVE_PIPE
 #define USE_pipe
 #else
@@ -161,20 +168,33 @@
 #define USE_NETWORKING
 #endif
 
-#ifdef HAVE_STAT64
+#if defined(HAVE_STAT64) && defined(HAVE_STRUCT_STAT64) && !(defined(__MACOSX__) || (defined(__APPLE__) && defined(__MACH__)))
 #define USE_stat
+#define ___struct_stat struct stat64
 #define ___stat stat64
 #define ___lstat lstat64
 #define ___fstat fstat64
 #else
 #ifdef HAVE_STAT
 #define USE_stat
+#define ___struct_stat struct stat
 #define ___stat stat
 #define ___lstat lstat
 #define ___fstat fstat
 #endif
 #endif
 
+#define USE_NONBLOCKING_FILE_IO
+
+#ifdef HAVE_TARGETCONDITIONALS_H
+#include <TargetConditionals.h>
+#ifdef TARGET_OS_IPHONE
+#if TARGET_OS_IPHONE == 1
+#undef USE_NONBLOCKING_FILE_IO
+#endif
+#endif
+#endif
+
 #ifdef HAVE_STRERROR
 #define USE_strerror
 #endif
@@ -187,6 +207,10 @@
 #define USE_sysconf
 #endif
 
+#ifdef HAVE_SYSCTL
+#define USE_sysctl
+#endif
+
 #ifdef HAVE_TCGETATTR
 #define USE_tcgetattr
 #endif
@@ -199,6 +223,18 @@
 #define USE_waitpid
 #endif
 
+#ifdef HAVE_MMAP
+#define USE_mmap
+#endif
+
+#if 0
+
+/*
+ * This code is now commented out as it seems to be causing trouble on
+ * CYGWIN and the problem it was trying to solve no longer seems to
+ * exist (perhaps the bug in CYGWIN has since been repaired).
+ */
+
 #ifdef __CYGWIN__
 /* 
  * Cygwin's timer implementation does not support ITIMER_VIRTUAL and
@@ -214,6 +250,8 @@
 
 #endif
 
+#endif
+
 
 #ifdef USE_WIN32
 
@@ -245,11 +283,14 @@
 #define USE_WSAGetLastError
 #define USE_GetConsoleWindow
 #define USE_GetModuleFileName
+#define USE_VirtualAlloc
+#define USE_GetSystemInfo
 
 #define HAVE_CLOCK 1
 #define HAVE_CREATETHREAD 1
 #define HAVE_GETPROCESSTIMES 1
 #define HAVE_GETSYSTEMTIME 1
+#define HAVE_GETSYSTEMINFO 1
 #define HAVE_LOADLIBRARY 1
 #define HAVE_MSGWAITFORMULTIPLEOBJECTS 1
 #define HAVE_Sleep 1
@@ -258,6 +299,7 @@
 #define HAVE_GETPEERNAME 1
 #define HAVE_GETSOCKNAME 1
 #undef HAVE_INET_PTON
+#define HAVE_GETADDRINFO 1
 #define HAVE_GETHOSTBYNAME 1
 #define HAVE_GETHOSTBYADDR 1
 #define HAVE_GETSERVBYNAME 1
@@ -451,6 +493,10 @@
 #define USE_inet_pton
 #endif
 
+#ifdef HAVE_GETADDRINFO
+#define USE_getaddrinfo
+#endif
+
 #ifdef HAVE_GETHOSTBYNAME
 #define USE_gethostbyname
 #endif
@@ -555,9 +601,17 @@
 #endif
 
 #ifdef USE_environ
+#ifdef HAVE_ENVIRON
 ___BEGIN_C_LINKAGE
 extern char **environ;
 ___END_C_LINKAGE
+#else
+#ifdef HAVE__NSGETENVIRON
+#define environ (*_NSGetEnviron())
+#undef INCLUDE_crt_externs_h
+#define INCLUDE_crt_externs_h
+#endif
+#endif
 #endif
 
 #ifdef USE_getenv
@@ -744,6 +798,13 @@ ___END_C_LINKAGE
 #define INCLUDE_sys_wait_h
 #endif
 
+#ifdef USE_mmap
+#undef INCLUDE_sys_types_h
+#define INCLUDE_sys_types_h
+#undef INCLUDE_sys_mman_h
+#define INCLUDE_sys_mman_h
+#endif
+
 #ifdef USE_gethostname
 #undef INCLUDE_unistd_h
 #define INCLUDE_unistd_h
@@ -770,6 +831,15 @@ ___END_C_LINKAGE
 #define INCLUDE_arpa_inet_h
 #endif
 
+#ifdef USE_getaddrinfo
+#undef INCLUDE_sys_types_h
+#define INCLUDE_sys_types_h
+#undef INCLUDE_sys_socket_h
+#define INCLUDE_sys_socket_h
+#undef INCLUDE_netdb_h
+#define INCLUDE_netdb_h
+#endif
+
 #ifdef USE_gethostbyname
 #undef INCLUDE_sys_socket_h
 #define INCLUDE_sys_socket_h
@@ -872,6 +942,27 @@ ___END_C_LINKAGE
 #define INCLUDE_util_h
 #endif
 
+#ifdef USE_getpt
+#undef INCLUDE_stdlib_h
+#define INCLUDE_stdlib_h
+#undef INCLUDE_stropts_h
+#define INCLUDE_stropts_h
+#endif
+
+#ifdef USE_ptsname
+#undef INCLUDE_stdlib_h
+#define INCLUDE_stdlib_h
+#undef INCLUDE_stropts_h
+#define INCLUDE_stropts_h
+#endif
+
+#ifdef USE_sysctl
+#undef INCLUDE_sys_types_h
+#define INCLUDE_sys_types_h
+#undef INCLUDE_sys_sysctl_h
+#define INCLUDE_sys_sysctl_h
+#endif
+
 #ifdef USE_tcgetattr
 #undef INCLUDE_termios_h
 #define INCLUDE_termios_h
@@ -889,7 +980,6 @@ ___END_C_LINKAGE
 #define INCLUDE_curses_h
 #endif
 
-
 /*---------------------------------------------------------------------------*/
 
 /* Inclusion of header files. */
@@ -999,6 +1089,12 @@ ___END_C_LINKAGE
 #endif
 #endif
 
+#ifdef INCLUDE_sys_mman_h
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif
+#endif
+
 #ifdef INCLUDE_stat_h
 #ifdef HAVE_STAT_H
 #include <stat.h>
@@ -1221,6 +1317,12 @@ typedef unsigned int fpu_control_t __attribute__ ((__mode__ (__HI__)));
 #endif
 #endif
 
+#ifdef INCLUDE_sys_sysctl_h
+#ifdef HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#endif
+#endif
+
 #ifdef INCLUDE_termios_h
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -1239,6 +1341,12 @@ typedef unsigned int fpu_control_t __attribute__ ((__mode__ (__HI__)));
 #endif
 #endif
 
+#ifdef INCLUDE_stropts_h
+#ifdef HAVE_STROPTS_H
+#include <stropts.h>
+#endif
+#endif
+
 #ifdef INCLUDE_libutil_h
 #ifdef HAVE_LIBUTIL_H
 #include <libutil.h>
@@ -1311,6 +1419,36 @@ typedef unsigned int fpu_control_t __attribute__ ((__mode__ (__HI__)));
 #endif
 #endif
 
+#ifdef INCLUDE_crt_externs_h
+#ifdef HAVE_CRT_EXTERNS_H
+#include <crt_externs.h>
+#endif
+#endif
+
+/*
+ * Use the process-time timer unless only the real-time timer is
+ * available (e.g. DJGPP).  Note that on some systems (e.g. MkLinux)
+ * ITIMER_VIRTUAL is an enum type, not a macro.
+ */
+
+#if defined(__CYGWIN__) || defined(__MACOSX__) || (defined(__APPLE__) && defined(__MACH__))
+/* ITIMER_VIRTUAL is broken under CYGWIN and MacOS X... use ITIMER_REAL */
+#undef ITIMER_VIRTUAL
+#endif
+
+#ifdef ITIMER_VIRTUAL
+#define HEARTBEAT_ITIMER ITIMER_VIRTUAL
+#define HEARTBEAT_SIG SIGVTALRM
+#else
+#ifdef ITIMER_REAL
+#define HEARTBEAT_ITIMER ITIMER_REAL
+#define HEARTBEAT_SIG SIGALRM
+#else
+#define HEARTBEAT_ITIMER ITIMER_VIRTUAL
+#define HEARTBEAT_SIG SIGVTALRM
+#endif
+#endif
+
 
 /*---------------------------------------------------------------------------*/
 
@@ -1340,6 +1478,16 @@ extern void ___enable_os_interrupts ___PVOID;
 
 /*---------------------------------------------------------------------------*/
 
+/* Processor information. */
+
+extern int ___processor_count ___PVOID;
+
+extern int ___processor_cache_size
+   ___P((___BOOL instruction_cache,
+         int level),
+        ());
+
+
 /* Virtual memory statistics. */
 
 extern void ___vm_stats
@@ -1364,6 +1512,13 @@ extern char *___os_system_type_string ___PVOID;
 extern char *___os_configure_command_string ___PVOID;
 
 
+/* C compilation environment information. */
+
+extern char *___os_obj_extension_string ___PVOID;
+extern char *___os_exe_extension_string ___PVOID;
+extern char *___os_bat_extension_string ___PVOID;
+
+
 /* OS initialization/finalization. */
 
 extern ___SCMOBJ ___setup_os ___PVOID;
