@@ -1,13 +1,13 @@
-/* File: "os_base.c", Time-stamp: <2009-03-18 09:25:14 feeley> */
+/* File: "os_base.c" */
 
-/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2012 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the most basic operating system services.
  */
 
 #define ___INCLUDED_FROM_OS_BASE
-#define ___VERSION 404004
+#define ___VERSION 406007
 #include "gambit.h"
 
 #include "os_base.h"
@@ -269,6 +269,83 @@ char *file;)
 #endif
 
 
+void *___alloc_mem_code
+   ___P((unsigned long bytes),
+        (bytes)
+unsigned long bytes;)
+{
+#ifndef USE_mmap
+#ifndef USE_VirtualAlloc
+
+  return NULL;
+
+#endif
+#endif
+
+#ifdef USE_mmap
+
+  ___BOOL executable = 1;
+
+  void* ptr = mmap (0,
+                    bytes + sizeof (long),
+                    PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0),
+                    MAP_PRIVATE | MAP_ANON,
+                    -1,
+                    0);
+
+  if (ptr == MAP_FAILED)
+    return NULL;
+
+  *___CAST(long*,ptr) = bytes;
+
+  return ___CAST(long*,ptr)+1;
+
+#endif
+
+#ifdef USE_VirtualAlloc
+
+  ___BOOL executable = 1;
+
+  void *ptr = VirtualAlloc (NULL,
+                            bytes,
+                            MEM_COMMIT,
+                            (executable
+                             ? PAGE_EXECUTE_READWRITE
+                             : PAGE_READWRITE));
+
+  return ptr;
+
+#endif
+}
+
+
+void ___free_mem_code
+   ___P((void *ptr),
+        (ptr)
+void *ptr;)
+{
+#ifndef USE_mmap
+#ifndef USE_VirtualAlloc
+
+#endif
+#endif
+
+#ifdef USE_mmap
+
+  long* p = ___CAST(long*,ptr)-1;
+
+  munmap (p, *p + sizeof (long));
+
+#endif
+
+#ifdef USE_VirtualAlloc
+
+  VirtualFree (ptr, 0, MEM_RELEASE);
+
+#endif
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 /* Program startup */
@@ -836,6 +913,61 @@ ___SCMOBJ ___err_code_from_h_errno ___PVOID
 #endif
 
 
+#ifdef USE_getaddrinfo
+
+
+___HIDDEN const char *gai_code_to_string
+   ___P((int code),
+        (code)
+int code;)
+{
+  return gai_strerror (code);
+}
+
+
+#ifdef ___DEBUG
+___SCMOBJ ___err_code_from_gai_code_debug
+   ___P((int code,
+         int lineno,
+         char *file),
+        (code,
+         lineno,
+         file)
+int code;
+int lineno;
+char *file;)
+#else
+___SCMOBJ ___err_code_from_gai_code
+   ___P((int code),
+        (code)
+int code;)
+#endif
+{
+  ___ERR_CODE e;
+
+#ifdef EAI_SYSTEM
+  if (code == EAI_SYSTEM)
+    e = err_code_from_errno ();
+  else
+#endif
+
+    e = ___GAI_CODE_ERR(code);
+
+#ifdef ___DEBUG
+  ___printf ("*** OS ERROR AT \"%s\"@%d.1 -- gai_code=%d (%s)\n",
+             file,
+             lineno,
+             code,
+             gai_code_to_string (code));
+#endif
+
+  return ___FIX(e);
+}
+
+
+#endif
+
+
 #ifdef USE_GetLastError
 
 
@@ -1209,6 +1341,21 @@ ___SCMOBJ err;)
 
       append_charstring (buf, &pos, msg);
 
+#endif
+    }
+  else if (facility >= ___ERR_CODE_FACILITY_GAI_CODE)
+    {
+      /* getaddressinfo error code */
+
+#ifdef USE_getaddrinfo
+
+      const char *msg = gai_code_to_string (___GAI_CODE_FROM_ERR_CODE(err_code));
+
+      if (msg == NULL)
+        msg = "Unknown error";
+
+      append_charstring (buf, &pos, msg);
+
 #endif
     }
   else
